/* These functions were ripped off from:
 * UNIX Network Programing, Volume 1, 2ed, by W Richard Stevens.
*/

#include <unistd.h>
#include <errno.h>

#include "ctalk.h"
#include "functions.h"

void output(int fd, void *msg)
{
   writen(fd, msg, strlen(msg));
}

ssize_t writen(int fd, const void *vptr, size_t n)
{
   size_t nleft;
   ssize_t nwritten;
   const char *ptr;

   ptr = vptr;
   nleft = n;
   while(nleft > 0)
   {
      if((nwritten = write(fd, ptr, nleft)) <= 0)
      {
         if(errno == EINTR) nwritten = 0;
         else return(-1);
      }
      nleft -= nwritten;
      ptr += nwritten;
   }
   return(n);
}

static ssize_t my_read(int fd, char *ptr)
{
   static int read_cnt = 0;
   static char *read_ptr;
   static char read_buf[MAXLINE];

   if(read_cnt <= 0)
   {
      again:
         if((read_cnt = read(fd, read_buf, sizeof(read_buf))) < 0) {
            if(errno == EINTR) goto again;
            return -1;
         } else if(read_cnt == 0) return 0;

         read_ptr = read_buf;
   }
   read_cnt--;
   *ptr = *read_ptr++;
   return(1);
}

ssize_t readline(int fd, void *vptr, size_t maxlen)
{
   ssize_t n, rc;
   char c, *ptr;

   ptr = vptr;
   for(n = 1; n < maxlen; n++)
   {
      if((rc = my_read(fd, &c)) == 1)
      {
         *ptr++ = c;
         if(c == '\n') break;
      } else if(rc == 0) {
         if(n == 1) return(0);
         else break;
      } else return(-1);
   }
   *ptr = 0;
   return (n);
}
         
